@@ -1,13 +1,17 @@
 from collections import deque
 from dataclasses import dataclass
 from datetime import datetime, timezone
+from decimal import Decimal
 from enum import Enum
 from pathlib import PurePath, PurePosixPath, PureWindowsPath
 from typing import Optional
 
 import pytest
+from fastapi._compat import PYDANTIC_V2
 from fastapi.encoders import jsonable_encoder
-from pydantic import BaseModel, Field, ValidationError, create_model
+from pydantic import BaseModel, Field, ValidationError
+
+from .utils import needs_pydanticv1, needs_pydanticv2
 
 
 class Person:
@@ -46,22 +50,6 @@ class Unserializable:
         raise NotImplementedError()
 
 
-class ModelWithCustomEncoder(BaseModel):
-    dt_field: datetime
-
-    class Config:
-        json_encoders = {
-            datetime: lambda dt: dt.replace(
-                microsecond=0, tzinfo=timezone.utc
-            ).isoformat()
-        }
-
-
-class ModelWithCustomEncoderSubclass(ModelWithCustomEncoder):
-    class Config:
-        pass
-
-
 class RoleEnum(Enum):
     admin = "admin"
     normal = "normal"
@@ -70,8 +58,12 @@ class RoleEnum(Enum):
 class ModelWithConfig(BaseModel):
     role: Optional[RoleEnum] = None
 
-    class Config:
-        use_enum_values = True
+    if PYDANTIC_V2:
+        model_config = {"use_enum_values": True}
+    else:
+
+        class Config:
+            use_enum_values = True
 
 
 class ModelWithAlias(BaseModel):
@@ -84,23 +76,6 @@ class ModelWithDefault(BaseModel):
     bla: str = "bla"
 
 
-class ModelWithRoot(BaseModel):
-    __root__: str
-
-
-@pytest.fixture(
-    name="model_with_path", params=[PurePath, PurePosixPath, PureWindowsPath]
-)
-def fixture_model_with_path(request):
-    class Config:
-        arbitrary_types_allowed = True
-
-    ModelWithPath = create_model(
-        "ModelWithPath", path=(request.param, ...), __config__=Config  # type: ignore
-    )
-    return ModelWithPath(path=request.param("/foo", "bar"))
-
-
 def test_encode_dict():
     pet = {"name": "Firulais", "owner": {"name": "Foo"}}
     assert jsonable_encoder(pet) == {"name": "Firulais", "owner": {"name": "Foo"}}
@@ -154,14 +129,47 @@ def test_encode_unsupported():
         jsonable_encoder(unserializable)
 
 
-def test_encode_custom_json_encoders_model():
+@needs_pydanticv2
+def test_encode_custom_json_encoders_model_pydanticv2():
+    from pydantic import field_serializer
+
+    class ModelWithCustomEncoder(BaseModel):
+        dt_field: datetime
+
+        @field_serializer("dt_field")
+        def serialize_dt_field(self, dt):
+            return dt.replace(microsecond=0, tzinfo=timezone.utc).isoformat()
+
+    class ModelWithCustomEncoderSubclass(ModelWithCustomEncoder):
+        pass
+
     model = ModelWithCustomEncoder(dt_field=datetime(2019, 1, 1, 8))
     assert jsonable_encoder(model) == {"dt_field": "2019-01-01T08:00:00+00:00"}
+    subclass_model = ModelWithCustomEncoderSubclass(dt_field=datetime(2019, 1, 1, 8))
+    assert jsonable_encoder(subclass_model) == {"dt_field": "2019-01-01T08:00:00+00:00"}
 
 
-def test_encode_custom_json_encoders_model_subclass():
-    model = ModelWithCustomEncoderSubclass(dt_field=datetime(2019, 1, 1, 8))
+# TODO: remove when deprecating Pydantic v1
+@needs_pydanticv1
+def test_encode_custom_json_encoders_model_pydanticv1():
+    class ModelWithCustomEncoder(BaseModel):
+        dt_field: datetime
+
+        class Config:
+            json_encoders = {
+                datetime: lambda dt: dt.replace(
+                    microsecond=0, tzinfo=timezone.utc
+                ).isoformat()
+            }
+
+    class ModelWithCustomEncoderSubclass(ModelWithCustomEncoder):
+        class Config:
+            pass
+
+    model = ModelWithCustomEncoder(dt_field=datetime(2019, 1, 1, 8))
     assert jsonable_encoder(model) == {"dt_field": "2019-01-01T08:00:00+00:00"}
+    subclass_model = ModelWithCustomEncoderSubclass(dt_field=datetime(2019, 1, 1, 8))
+    assert jsonable_encoder(subclass_model) == {"dt_field": "2019-01-01T08:00:00+00:00"}
 
 
 def test_encode_model_with_config():
@@ -197,6 +205,7 @@ def test_encode_model_with_default():
     }
 
 
+@needs_pydanticv1
 def test_custom_encoders():
     class safe_datetime(datetime):
         pass
@@ -227,19 +236,73 @@ def test_custom_enum_encoders():
     assert encoded_instance == custom_enum_encoder(instance)
 
 
-def test_encode_model_with_path(model_with_path):
-    if isinstance(model_with_path.path, PureWindowsPath):
-        expected = "\\foo\\bar"
-    else:
-        expected = "/foo/bar"
-    assert jsonable_encoder(model_with_path) == {"path": expected}
+def test_encode_model_with_pure_path():
+    class ModelWithPath(BaseModel):
+        path: PurePath
+
+        if PYDANTIC_V2:
+            model_config = {"arbitrary_types_allowed": True}
+        else:
+
+            class Config:
+                arbitrary_types_allowed = True
+
+    test_path = PurePath("/foo", "bar")
+    obj = ModelWithPath(path=test_path)
+    assert jsonable_encoder(obj) == {"path": str(test_path)}
 
 
+def test_encode_model_with_pure_posix_path():
+    class ModelWithPath(BaseModel):
+        path: PurePosixPath
+
+        if PYDANTIC_V2:
+            model_config = {"arbitrary_types_allowed": True}
+        else:
+
+            class Config:
+                arbitrary_types_allowed = True
+
+    obj = ModelWithPath(path=PurePosixPath("/foo", "bar"))
+    assert jsonable_encoder(obj) == {"path": "/foo/bar"}
+
+
+def test_encode_model_with_pure_windows_path():
+    class ModelWithPath(BaseModel):
+        path: PureWindowsPath
+
+        if PYDANTIC_V2:
+            model_config = {"arbitrary_types_allowed": True}
+        else:
+
+            class Config:
+                arbitrary_types_allowed = True
+
+    obj = ModelWithPath(path=PureWindowsPath("/foo", "bar"))
+    assert jsonable_encoder(obj) == {"path": "\\foo\\bar"}
+
+
+@needs_pydanticv1
 def test_encode_root():
+    class ModelWithRoot(BaseModel):
+        __root__: str
+
     model = ModelWithRoot(__root__="Foo")
     assert jsonable_encoder(model) == "Foo"
 
 
+@needs_pydanticv2
+def test_decimal_encoder_float():
+    data = {"value": Decimal(1.23)}
+    assert jsonable_encoder(data) == {"value": 1.23}
+
+
+@needs_pydanticv2
+def test_decimal_encoder_int():
+    data = {"value": Decimal(2)}
+    assert jsonable_encoder(data) == {"value": 2}
+
+
 def test_encode_deque_encodes_child_models():
     class Model(BaseModel):
         test: str
